@@ -3,45 +3,15 @@ from asyncio import AbstractEventLoop, get_event_loop
 
 import uuid
 import warnings
+from datetime import datetime
+from typing import Tuple, Mapping, Any, Callable, Union, Iterable, Type
 
-try:
-    import betterproto
-except:
-    betterproto = None
-
-from google.protobuf.reflection import GeneratedProtocolMessageType
-from google.protobuf.struct_pb2 import Struct
-from google.protobuf.wrappers_pb2 import BytesValue, StringValue
-from typing import Tuple, Mapping, Any, Callable, Union, Iterable
-
-from .. import Task
+from .serializers.base import BaseTaskSerializer
+from .serializers.betterproto import BetterprotoSerializer
+from .task import Task, TaskMetadata, TaskDecodeError
 from ..component import Component
 
 
-class TaskDecodeError(ValueError):
-    """
-    Received message cannot be decoded into task.
-    """
-
-    def __init__(self, payload: bytes, meta: Mapping, exc: Exception, handled: bool):
-        """
-        :param payload: Received payload
-        :param meta:    Received meta
-        :param exc:     Raised exception
-        :param handled: True if exception has been handled by client and this is just a notification exception,
-                        handled means that task is not lost on the queue. If False, this case is not handled
-                        and worker should take care of logging and administrator notification in the same
-                        way as other task processing messages are handled.
-        """
-        self.payload = payload
-        self.meta = meta
-        self.exception = exc
-        self.is_handled = handled
-
-    def __str__(self):
-        return ("Handled" if self.is_handled else "Unhandled") + f" {self.__class__.__name__}({self.exception})"
-
-
 class PatchworkPublisher(Component):
     """
     Common code for sync and async publishers
@@ -53,6 +23,7 @@ class PatchworkPublisher(Component):
         :cvar max_message_size:     Maximum allowed message size (on the wire) in bytes
         """
         max_message_size: int = 1024*1024       # 1MB
+        serializer: Type[BaseTaskSerializer] = BetterprotoSerializer
 
     settings: Config
     asynchronous: bool
@@ -61,6 +32,7 @@ class PatchworkPublisher(Component):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.routing = []
+        self.task_serializer = self.settings.serializer()
 
     def __repr__(self):
         return super().__repr__().replace('Component', 'Publisher')
@@ -89,7 +61,7 @@ class PatchworkPublisher(Component):
         :param task:    Task instance
         :return:    Serialized task
         """
-        return task.SerializeToString()
+        return self.task_serializer.encode(task)
 
     def _prepare_task(self, task: Task) -> bytes:
         """
@@ -101,9 +73,9 @@ class PatchworkPublisher(Component):
         """
         if not task.uuid:
             task.uuid = str(uuid.uuid4())
-        task.meta.scheduled.GetCurrentTime()
+        task.meta.scheduled = datetime.now()
 
-        if task.meta.HasField('expires') and task.meta.expires < task.meta.scheduled:
+        if task.meta.expires and task.meta.expires < task.meta.scheduled:
             raise ValueError('unable to send task which is already expired')
 
         if not task.meta.queue_name:
@@ -118,43 +90,35 @@ class PatchworkPublisher(Component):
 
         return payload
 
-    def _build_task(self, payload: Any, meta: Mapping, cause: Task = None, task_type: str = None):
-        meta = dict(meta)
-        task = Task()
-        task.uuid = str(uuid.uuid4())
-        task.task_type = task_type or payload.__class__.__name__
+    def _build_task(self, payload: Any, meta: Union[Mapping, TaskMetadata], cause: Task = None, task_type: str = None):
+        if not isinstance(meta, TaskMetadata):
+            meta = dict(meta)
+            metadata = TaskMetadata()
+
+            if 'not_before' in meta:
+                metadata.not_before = meta.pop('not_before')
+            if 'expires' in meta:
+                metadata.expires = meta.pop('expires')
+            if 'max_retries' in meta:
+                metadata.max_retries = meta.pop('max_retries')
+            if 'queue_name' in meta:
+                metadata.queue_name = meta.pop('queue_name')
+
+            if meta:
+                metadata.extra = dict(meta)
+        else:
+            metadata = meta
+
+        task = Task(
+            uuid=uuid.uuid4(),
+            meta=metadata,
+            task_type=task_type or payload.__class__.__name__,
+            payload=payload
+        )
 
         if cause is not None:
             task.correlation_id = cause.correlation_id or cause.uuid
 
-        if isinstance(payload, str):
-            wrapper = StringValue(value=payload)
-            task.payload.pack(wrapper)
-        elif isinstance(payload, bytes):
-            wrapper = BytesValue(value=payload)
-            task.payload.Pack(wrapper)
-        elif isinstance(payload, GeneratedProtocolMessageType):
-            task.payload.Pack(payload)
-        elif betterproto is not None and isinstance(payload, betterproto.Message):
-            task.payload.type_url = f'betterproto.{payload.__class__.__name__}'
-            task.payload.value = bytes(payload)
-        else:
-            raise ValueError('unsupported payload type')
-
-        if 'not_before' in meta:
-            task.meta.not_before = meta.pop('not_before')
-        if 'expires' in meta:
-            task.meta.expires = meta.pop('expires')
-        if 'max_retires' in meta:
-            task.meta.max_retries = meta.pop('max_retries')
-        if 'queue_name' in meta:
-            task.meta.queue_name = meta.pop('queue_name')
-
-        if meta:
-            extra = Struct()
-            extra.update(meta)
-            task.meta.extra.Pack(extra)
-
         return task
 
     def __del__(self):
@@ -164,8 +128,19 @@ class PatchworkPublisher(Component):
 
 class PatchworkSubscriber(Component):
 
+    class Config(Component.Config):
+        """
+        Settings schema for each Patchwork Client.
+        """
+        serializer: Type[BaseTaskSerializer] = BetterprotoSerializer
+
+    settings: Config
     asynchronous: bool
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.task_serializer = self.settings.serializer()
+
     def __repr__(self):
         return super().__repr__().replace('Component', 'Subscriber')
 
@@ -176,9 +151,7 @@ class PatchworkSubscriber(Component):
         :param meta:    Additional data passed by receiver
         :return:        Deserialized task instance
         """
-        task = Task()
-        task.ParseFromString(payload)
-        return task
+        return self.task_serializer.decode(payload)
 
     def _process_received_task(self, payload: bytes, meta: Mapping) -> Task:
         """
@@ -193,7 +166,7 @@ class PatchworkSubscriber(Component):
             handled = self._handle_deserialize_error(payload, meta, exc)
             raise TaskDecodeError(payload, meta, exc, handled) from exc
 
-        task.meta.received.GetCurrentTime()
+        task.meta.received = datetime.now()
         task.meta.queue_name = meta.get('queue_name', None)
         return task
 
