@@ -1,9 +1,5 @@
 # -*- coding: utf-8 -*-
 
-"""Created on 16.04.2020
-
-.. moduleauthor:: Paweł Pecio
-"""
 import json
 from typing import Any
 
